# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
__version__ = "3.3.0"
SERVER_VERSION = "4.6.0"
